import os
import sys
import zipfile

try:
    import shutil
except ImportError:
    print('Error!\nInstall python-shutil to build')
    sys.exit(1)


BUILD_OUTPUT_ROOT = '/tmp/'
OUTPUT_FOLDER_NAME = 'OctoPrint-3DPrinterOS'
PLUGIN_SRC_FOLDER_NAME = 'octoprint_3dprinteros'
BUILD_OUTPUT_FOLDER = os.path.join(BUILD_OUTPUT_ROOT, OUTPUT_FOLDER_NAME)
THIS_FOLDER = os.path.dirname(os.path.abspath(__file__))

DONT_COPY = (".git", ".idea", "__pycache__", "*.pyc", "_*", ".gitignore", ".pylintrc")
USED_PRINTEROS_MODULES = ['paths.py', 'config.py', 'log.py', 'platforms.py', 'version.py', 'client_ssl_context.py', 'http_client.py', 'awaitable.py', 'user_login.py',
                         'app.py', 'slave_app.py', 'downloader.py', 'printer_interface.py', 'printer_states.py', 'printer_settings_and_id.py', 'forced_settings.py',
                         'base_sender.py', 'octoprint_sender.py', 'octoprint_3dprinteros.py', 'camera_controller.py', 'no_subproc_camera_controller.py',
                         'base_detector.py', 'static_and_stored_detect.py', 'default_settings.json', 'default_printer_profiles.json', 'dual_cam.py', 'http_snap_cam.py']

def copy_file(in_file_path, out_file_path):
    with open(in_file_path, "rb") as in_file:
        with open(out_file_path, "wb") as out_file:
            out_file.write(in_file.read())

def copy_needed_file_to_plugin_out():
    if os.path.isdir(BUILD_OUTPUT_FOLDER):
        print('Removing output folder: ' + BUILD_OUTPUT_FOLDER)
        shutil.rmtree(BUILD_OUTPUT_FOLDER)
    print(f'Copying {os.path.abspath(THIS_FOLDER)} to output folder: ' + BUILD_OUTPUT_FOLDER)
    shutil.copytree(THIS_FOLDER, BUILD_OUTPUT_FOLDER, ignore=lambda x, y: DONT_COPY)
    dst_path = os.path.join(BUILD_OUTPUT_FOLDER, PLUGIN_SRC_FOLDER_NAME)
    for filename in USED_PRINTEROS_MODULES:
        src_path = os.path.join(THIS_FOLDER, "..", filename)
        print(f'Copying {os.path.abspath(src_path)} to output folder: ' + dst_path)
        shutil.copy(src_path, dst_path)

def put_version():
    version_filename = os.path.join(THIS_FOLDER, "..", 'version.py')
    with open(version_filename) as f:
        for line in f:
            if line.startswith('version'):
                version = line.split('=')[1].strip().strip('"').strip("'")
                break
        else:
            print('Error! Unable to parse client version')
            sys.exit(1)
        filepath = os.path.join(BUILD_OUTPUT_FOLDER, 'setup.py')
        with open(filepath, 'r+') as f:
            text = f.read().replace('0.0.0', version)
            f.seek(0)
            f.write(text)
        print('Setting version in setup.py to ' + version)

def zip_plugin():
    zip_filepath = os.path.join(BUILD_OUTPUT_ROOT, OUTPUT_FOLDER_NAME + '.zip')
    with zipfile.ZipFile(zip_filepath, "w", zipfile.ZIP_DEFLATED) as zf:
        for root, _, files in os.walk(BUILD_OUTPUT_FOLDER):
            for file in files:
                for excl in DONT_COPY:
                    if excl in root.split('/') or file == excl:
                        break
                else:
                    filename = os.path.join(root, file)
                    arcname = os.path.relpath(os.path.join(root, file), BUILD_OUTPUT_FOLDER)
                    print('Zipping %s as %s' % (filename, arcname))
                    zf.write(filename, arcname=arcname)

if __name__ == "__main__":
    copy_needed_file_to_plugin_out() 
    put_version()
    zip_plugin()
    print('Done!')
